; block.asm
;
; Robert stling, 2002-03-09
;
; Block devices are accessed through these functions.

%define _block_asm 0

[bits 32]
[section .text]


%include "block.inc"
%include "mem.inc"

%include "llist.inc"

global	block_add_device
global	block_get_pointer
global	block_call


; block_add_device
;
; Add a block device driver.
;
; ecx = pointer to list of function pointers and driver ID word
;	ID
;	open
;	close
;	read
;	write
;	ioctl
;
; returns:
; eax = 0 if OK, else 1

block_add_device:
		push	esi
		push	edi

		sllist_create block_struct, block_device_list
		or	eax,eax
		jnz	.error

		lea	edi,[esi+block_struct.id]	; edi points to the first of the 6 data words
		mov	esi,ecx				; esi points to a list with these 6 words (given by caller)
		movsd	; ID				; copy the data...
		movsd	; open
		movsd	; close
		movsd	; read
		movsd	; write
		movsd	; ioctl
.error:

		pop	edi
		pop	esi
		ret



; block_get_pointer
;
; Get pointer to device struct from device ID
;
; ecx = driver ID
;
; returns:
; eax = pointer to struct, 0 if error

block_get_pointer:
		push	edx

		mov	eax,dword[block_device_list]	; eax points to the element to try
		mov	edx,eax				; ..and edx to the previous element
.try_next:
		or	eax,eax				; have we found the last one?
		jz	.not_found			; yes... the searched element was not found.

		mov	edx,eax
		mov	eax,dword[eax+block_struct.next]	; load the next block

		cmp	dword[edx+block_struct.id],ecx	; is this the right element?
		jz	.found				; yay!
		jmp	short .try_next			; nope, try again

.found:							; edx now points to the element wanted
		xchg	eax,edx				; ..now eax does
.not_found:						; eax is either 0, or the pointer to the element
		pop	edx
		ret



; block_call
;
; Call a function a block device driver provides.
;
; ecx = driver ID
; edx = call type
;	block_open
;	block_close
;	block_read
;	block_write
;	block_ioctl
;
; Other registers (except eax) are parameters to the call.
;
; returns:
; eax = 0 if OK, 1 if not

block_call:
		call	block_get_pointer		; get pointer to this block
		or	eax,eax				; was it found?
		jz	.not_found			; no...

		call	dword[eax+edx]

		mov	eax,1
.not_found:
		xor	al,1
		ret



[section .data]

block_device_list dd	0


[section .bss]
