; ramdisk.asm
;
; Robert stling, 2002-03-15
;
; Simple RAM disk driver.

%define _ramdisk_asm 0

[bits 32]
[section .text]

	struct ramdisk_struct
.size		resd	1				; size of disk in 512-byte sectors
.ptr		resd	1				; pointer to the data
	endstruc

%include "ramdisk.inc"
%include "block.inc"
%include "mem.inc"

; ramdisk_create
;
; ebx = size in sectors
;
; Returns:
; eax = 0 if OK, else 1
; esi = pointer to ramdisk_struct

ramdisk_create:
		push	edi

		mov	eax,ebx				; eax = ebx = size in sectors
		shl	eax,9				; multiply by 512 (sector size)
		call	mem_alloc			; try to allocate some memory
		or	esi,esi				; did it return 0?
		jz	.error				; yes, quit

		mov	edi,esi				; save pointer to memory in edi

		mov	eax,ramdisk_struct_size
		call	mem_alloc			; allocate memory for our struct
		or	esi,esi				; error?
		jz	.error				; yes, return

		mov	dword[esi+ramdisk_struct.ptr],edi	; save pointer to disk in RAM
		mov	dword[esi+ramdisk_struct.size],ebx	; save size of disk (in sectors)

		xor	eax,eax
		jmp	short .quit
.error:
		mov	eax,1
.quit:
		pop	edi
		ret


[section .data]


[section .bss]
