
; BozOS boot code - 0.12
; Used by the Primula operating system
;
; Written by Robert stling - robost86@hotmail.com
;
; http://primula.sourceforge.net/
; http://bozos.sourceforge.net/
;
; Licensed under the BSD license

; German translation by Mathias Bandt  - bandi@greifswald-online.de
; French translation by pierre ancelot - pierreact@hotmail.com
; Swedish translation by me, Robert

; The purpose of this code is to load the boot loader. Any advanced
; functions, like booting multiple OSs, graphics etc. should be in
; the boot loader code, and not here.

; 2002-02-02:
; Modifying for compatibility with the EBFS file system.

; To do:
;	* Better error checking

%INCLUDE 'config.inc'
%INCLUDE 'bootsector.inc'

		[BITS	16]

		ORG	0X7C30
		; REMEMBER THAT WE ARE LOADED TO 0X07C0:0X0030

		MOV	AX,3
		INT	0X10			; INIT MODE 0X03

		XOR	AX,AX
		MOV	DS,AX			; DS = 0X0000
		MOV	ES,AX			; ES = 0X0000
		DB	0XEA
		DW	NEW_CS			; IP = NEW_CS
		DW	0X0000			; CS = 0X0000

NEW_CS:		CALL	PRINTDIR		; PRINT BOOT MESSAGE 
		DB	0X0D,0X0A

%IF LANGUAGE = ENGLISH
                DB      "Loading Primula...",0x0D,0X0A,0
%ENDIF
%IF LANGUAGE = SWEDISH
                DB      "Laddar Primula...",0X0D,0X0A,0
%ENDIF
%IF LANGUAGE = GERMAN
                DB      "Lade Primula...",0X0D,0X0A,0
%ENDIF
%IF LANGUAGE = FRENCH
                DB      "Chargement de Primula",0X0D,0X0A,0
%ENDIF

		MOV	AX,WORD[0X7C00+BOOT_BOOTLOADER]
		MOV	DX,WORD[0X7C00+BOOT_BOOTLOADER+2]
		ADD	AX,WORD[0X7C00+BOOT_BASESECTOR]
		ADC	DX,WORD[0X7C00+BOOT_BASESECTOR+2]
		MOV	CL,BYTE[0X7C00+BOOT_HEADS]	; DX:AX = BOOT LOADER SECTOR
		XOR	CH,CH			; CX = HEADS
		DIV	CX
		; AH = HEAD, DX = SECTOR+(SECTORS_PER_TRACK*TRACK)
		MOV	BL,AL			; BL = HEAD

		MOV	AX,DX
		XOR	DX,DX
		MOV	CX,WORD[0X7C00+BOOT_SECTORS]
		DIV	CX
		INC	DX
		; AX = TRACK, DX (ONLY DL USED) = SECTOR

		CMP	BYTE[0X7C00+BOOT_DISK],0X80	; IF < 0X80, IT'S A FLOPPY
		JB	FD
		SHL	AX,6			; AX = (AX<<6)|DL (BIOS
		OR	AL,DL			; TRACK/SECTOR NR)
		MOV	CX,AX			; CX = TRACK/SECTOR
		JMP	SHORT HD
FD:		MOV	CH,AL
		MOV	CL,DL			; CX = TRACK/SECTOR
HD:		MOV	DH,BL			; DH = HEAD
		MOV	DL,BYTE[0X7C00+BOOT_DISK] ; DL = DISK

		MOV	BX,0X7E00

		MOV	AH,0X02			; READ SECTORS
		MOV	AL,BYTE[0X7C00+BOOT_BOOTLOADERSIZE]
.TRY:		INT	0X13
		JC	.TRY			; ON ERROR, TRY AGAIN

		DB	0XEA
		DW	0X7E00
		DW	0X0000


PRINTDIR:	POP	SI			; GET THE RETURN ADDRESS
		MOV	AH,0X0E			; (INT 0X10/0XE: PRINT CHAR)
.L0:		LODSB				; GET A CHAR
		OR	AL,AL			; IF IT'S 0
		JZ	.L1			; QUIT
		INT	0X10			; PRINT THE CHAR
		JMP	SHORT .L0		; GO FOR THE NEXT CHAR
.L1:		PUSH	SI			; QUIT: PUSH THE OFFSET
		RET				; AFTER THE 0, AND RETURN
