
; Primula bootloader - 0.2
; Written by Robert stling
;
; Based on the BozOS boot loader by Robert stling
;
; http://primula.sourceforge.net/
; http://bozos.sourceforge.net/
;
; This code should load the kernel (from the file system on the disk) 
; and launch it. It's loaded by the boot code to 0x0000:0x7e00
;
; Written by Robert stling           - robost86@hotmail.com
; German translation by Mathias Bandt   - bandi@greifswald-online.de
; French translation by Pierre Ancelot  - pierreact@hotmail.com
; Swedish translation by me, Robert

; Changelog:

; Robert stling 2002-02-02:
; * Modified for the EBFS file system.
; * Name change to Primula (duh...)

; Robert stling 2001-12-13:
; * Licensed under the BSD license.
; * Name changed to Infinite.

; Robert stling 2001-12-09:
; * Got the elf loader to work.

; Robert stling 2001-12-08:
; * Started working on the BlueOS project, making this set up protected mode :)

; Pierre Ancelot 2001-12-07:
; * French translation added :)

; Robert stling 2001-10-29:
; * Now you can choose if you want the logo in include/config.inc

; Robert stling 2001-10-05:
; * Bandis german translation added :)

; Robert stling 2001-09-16:
; * Wohooo! Finally go it to work :)

; Robert stling 2001-09-16:
; * Added some comments, changes some small stuff.

; To do:

; * Support for kernel located over the 8MB in the FS
; * Optimization, bug-fixing & other fun stuff ;-)

%INCLUDE 'config.inc'

		ORG	0X7E00

; WE ASSUME DS = ES = CS = 0, IP = 0X7E00
; IF NOT, SOMEONE'S FUCKED WITH THE BOOT LOADER

MAIN:		
		MOV	SI,STR_LOADING_KERNEL
		CALL	PRINT

		MOV	AX,TMP_SEGMENT
		MOV	ES,AX		; LOAD FILE TABLE TO
		XOR	BX,BX		; TMP_SEGMENT:0X0000

		MOV	AX,WORD[0X7C00+BOOT_FILETABLE]
		MOV	DX,WORD[0X7C00+BOOT_FILETABLE+2]
		ADD	AX,WORD[0X7C00+BOOT_BASESECTOR]
		ADC	DX,WORD[0X7C00+BOOT_BASESECTOR+2]
		; DX:AX IS FILE TABLE SECTOR

		MOV	DI,WORD[0X7C00+BOOT_FILETABLESIZE]
		AND	DI,0X00FF	; DI = SIZE OF FILE TABLE

		MOV	CL,BYTE[0X7C00+BOOT_DISK]

		CALL	READ_SECTORS

		MOV	DI,FILE_NAME	; FILE NAME FIELD OF THE FILE ENTRY
					; STRUCTURE

		MOV	CL,BYTE[0X7C00+BOOT_FILETABLESIZE]

		XOR	CH,CH		; LET'S SEARCH IN FILETABLESIZE * 8
		SHL	CX,3		; FILE ENTRIES


.NEXTFILE:	PUSH	DI
		PUSH	CX

		MOV	SI,KERNEL_NAME	; FILE NAME OF KERNEL
		MOV	CX,0X28		; FILE NAME SIZE
		CALL	STRNCMP		; COMPARE
		OR	AL,AL
		JZ	.FILEFOUND	; IF FOUND, GOTO .FILEFOUND

		POP	CX		; CX = NR OF ENTRIES LEFT
		POP	DI		; ES:DI POINTS TO FILE NAME
		ADD	DI,0X40		; NEXT FILE ENTRY
		LOOP	.NEXTFILE	; IF NOT FOUND, SEARCH NEXT ENTRY

		MOV	SI,STR_NO_KERNEL
		CALL	PRINT		; ARGHH! NO KERNEL FOUND.

.L0		JMP	SHORT .L0	; WAIT UNTIL REBOOT...

.FILEFOUND:	POP	CX
		POP	DI
		POP	SI
		SUB	DI,FILE_NAME	; DI = PTR TO FILE ENTRY

		MOV	SI,STR_KERNEL_FOUND
		CALL	PRINT		; KERNEL FOUND =)

		MOV	AX,WORD[ES:DI+FILE_SIZE]
		MOV	WORD[KERNEL_SIZE],AX
		MOV	AX,WORD[ES:DI+FILE_SIZE+2]
		MOV	WORD[KERNEL_SIZE+2],AX

		MOV	AX,WORD[ES:DI+FILE_SECTOR]
		MOV	DX,WORD[ES:DI+FILE_SECTOR+2]
		; DX:AX IS THE FILES BASE SECTOR

		PUSH	AX
		PUSH	DX		; SAVE THAT VALUE

		MOV	DI,WORD[0X7C00+BOOT_SIZE]
		SHR	DI,7
		MOV	DX,WORD[0X7C00+BOOT_SIZE+2]
		SHL	DX,7
		OR	DI,DX
		INC	DI		; DI = BOOT_SIZE >> 7

		MOV	AX,WORD[0X7C00+BOOT_SECTORTABLE]
		MOV	DX,WORD[0X7C00+BOOT_SECTORTABLE+2]
		ADD	AX,WORD[0X7C00+BOOT_BASESECTOR]
		ADC	DX,WORD[0X7C00+BOOT_BASESECTOR+2]
		; DX:AX IS THE SECTOR TABLE SECTOR

		MOV	CL,BYTE[0X7C00+BOOT_DISK]	; FROM THIS DISK
		XOR	BX,BX		; TO: TMP_SEGMENT:0

		CALL	READ_SECTORS	; READ THE SECTORS

		MOV	SI,STR_SECTOR_TABLE_LOADED
		CALL	PRINT

		MOV	CX,KERNEL_TMP_SEGMENT
		MOV	ES,CX		; ES = KERNEL_SEGMENT

		POP	DX
		POP	AX		; DX:AX IS FILE BASE SECTOR

		XOR	BX,BX		; KERNEL_SEGMENT:0

.NEXTSECTOR:	PUSH	BX

		MOV	CL,BYTE[0X7C00+BOOT_DISK]	; READ FROM THIS DISK
		MOV	DI,1		; READ 1 SECTOR

		ADD	AX,WORD[0X7C00+BOOT_BASESECTOR]
		ADC	DX,WORD[0X7C00+BOOT_BASESECTOR+2]
		PUSH	AX
		PUSH	DX
		CALL	READ_SECTORS	; READ THE SECTOR
		POP	DX
		POP	AX

		POP	BX
		ADD	BX,0X0200	; ES:BI += 0X200

		PUSH	ES
		PUSH	DX
		MOV	DI,AX		; NOTE: THIS CODE NEEDS THE KERNEL
		SHL	DI,2		; TO BE LOCATED WITHIN THE FIRST
					; 8 MB ON THE DISK
		MOV	DX,TMP_SEGMENT
		MOV	ES,DX		; ES = TMP_SEGMENT
		POP	DX

		MOV	AX,WORD[ES:DI]	; DX:AX = NEW SECTOR VALUE
		MOV	DX,WORD[ES:DI+2]
		POP	ES		; ES = KERNEL_SEGMENT

		CMP	DX,0XFFFF
		JNZ	.NEXTSECTOR	; WHILE SECTOR_VALUE != SECTOR_LAST
		CMP	AX,SECTOR_LAST
		JNZ	.NEXTSECTOR

		MOV	SI,STR_KERNEL_LOADED
		CALL	PRINT		; KERNEL IS LOADED

		CLI
		LGDT	[GDT_TABLE]

		MOV	ESP,0X9FFFF

		MOV	EAX,CR0
		OR	AL,1
		MOV	CR0,EAX

		DB	0XEA
		DW	START32,8

		[BITS 32]
START32:

		MOV	AX,0X10
		MOV	DS,AX
		MOV	ES,AX
		MOV	FS,AX
		MOV	GS,AX
		MOV	SS,AX

		CALL	ENABLE_A20

		MOV	ESP,0X9FFFF		; PUT STACK IN TOP OF LOW MEM

		MOV	DX,0X03F2
		MOV	AL,0X0C
		OUT	DX,AL

		MOV	ESI,KERNEL_TMP_SEGMENT<<4

		CMP	DWORD[ESI],0X464C457F	; ELF SIGNATURE
		JNZ	.ELF_ERROR
		CMP	WORD[ESI+4],0X0101	; MUST BE 32-BIT, AND LSB ENODING
		JNZ	.ELF_ERROR
		CMP	BYTE[ESI+6],0X01	; 'FILE VERSION' MUST BE VALID
		JNZ	.ELF_ERROR

		JMP	SHORT .ELF_OK
.ELF_ERROR:
		JMP	INF_LOOP
.ELF_OK:

		MOV	EAX,DWORD[ESI+0X18]
		MOV	DWORD[KERNEL_ENTRY],EAX

		MOVZX	ECX,WORD[ESI+0X2C]	; ECX = NUMBER OF SECTIONS

LOAD_SECTION:
		DEC	CX
		PUSH	CX

		MOVZX	EAX,WORD[ESI+0X2A]	; PROGRAM HEADER ENTRY SIZE
		MUL	CX			; SECTION NR

		MOV	EBX,DWORD[ESI+0X1C]	; PROGRAM HEADER FILE OFFSET
		ADD	EBX,EAX

		CMP	DWORD[ESI+EBX],1	; SHOULD WE LOAD THIS?
		JNZ	NEXT_SECTION		; NO

		MOV	ECX,DWORD[ESI+EBX+4]	; OFFSET OF SEGMENT IN ELF FILE
		MOV	EBP,DWORD[ESI+EBX+0X10]	; SIZE OF SEGMENT IN ELF FILE
		MOV	EDI,DWORD[ESI+EBX+8]	; MEMORY ADDRESS OF SECTION
		MOV	EAX,DWORD[ESI+EBX+0X14]	; EBX = SIZE OF THE SECTION IN MEM
		MOV	EBX,EAX

		PUSH	EBP
		PUSH	ECX
		PUSH	ESI
		PUSH	EDI

		ADD	ESI,ECX
		MOV	ECX,EBP
		REP	MOVSB

		POP	EDI
		POP	ESI
		POP	ECX
		POP	EAX

		SUB	EBX,EAX
		JZ	NEXT_SECTION

		ADD	EDI,EAX
		MOV	AL,0
		MOV	ECX,EBX
		REP	STOSB		; MEMSET(EAX+EDI,0,EBX=ECX)

NEXT_SECTION:
		POP	CX
		OR	CX,CX
		JNZ	LOAD_SECTION

		MOV	EAX,2
		PUSH	EAX
		POPFD

		MOV	EAX,DWORD[KERNEL_ENTRY]
		CALL	EAX


INF_LOOP:	JMP	SHORT INF_LOOP

; ***** MISC. USEFUL FUNCTIONS *****

		[BITS 32]

ENABLE_A20:
		MOV	AL,0XD1
		OUT	0X64,AL
		CALL	A20WAIT

		MOV	AL,0XDF
		OUT	0X60,AL
		CALL	A20WAIT

		MOV	AL,0XFF
		OUT	0X64,AL
		CALL	A20WAIT

		RET

A20WAIT:
		IN	AL,0X64
		JMP	.L0
.L0:		AND	AL,2
		JNZ	A20WAIT
		RET

		[BITS 16]

; COMPARE DS:SI WITH ES:DI, MAX STR LEN IS CX BYTES
STRNCMP:	LODSB			; AL = S1[I]
		OR	AL,AL		; IF AL = 0
		JZ	.L0		; QUIT
		CMP	AL,BYTE[ES:DI]	; COMPARE AL WITH [ES:DI]
		JNZ	.L0		; QUIT IF NOT EQUAL
		INC	DI		; DI++
		LOOP	STRNCMP		; LOOP CX TIMES
		XOR	AL,AL		; CX = 0, STRINGS EQUAL
		RET
.L0:		SUB	AL,BYTE[ES:DI]	; COMPARE THE LAST CHARS
		RET

; PRINT DS:SI TO SCREEEN
PRINT:		MOV	AH,0X0E		; PRINT DS:SI
		XOR	BX,BX
.L0:		LODSB
		OR	AL,AL
		JZ	.L1
		INT	0X10
		JMP	SHORT .L0
.L1:		RET

; PRINT VALUE IN AX
PRINTHEXW:	XCHG	AL,AH
		CALL	PRINTHEX
		XCHG	AL,AH
		CALL	PRINTHEX
		RET

; PRINT HEX VALUE IN AL
PRINTHEX:	PUSH	BX
		PUSH	AX
		XOR	BH,BH
		MOV	BL,AL
		SHR	BL,4		; BX = AL >> 4
		ADD	BX,HEXTABLE	; DS:BX POINTS TO CHAR
		MOV	AH,0X0E
		MOV	AL,BYTE[BX]
		INT	0X10
		POP	AX
		PUSH	AX
		XOR	BH,BH
		MOV	BL,AL
		AND	BL,0X0F		; BX = AL & 0X0F
		ADD	BX,HEXTABLE
		MOV	AH,0X0E
		MOV	AL,BYTE[BX]
		INT	0X10
		POP	AX
		POP	BX
		RET

READ_SECTORS:	PUSH	AX		; DX:AX = SECTOR, DI = COUNT
		PUSH	DX		; ES:BX = BUFFER, CL = DISK
		PUSH	CX
		PUSH	BX

		PUSH	CX
		CALL	LONG_TO_BIOS	; CONVERT DX:AX TO BIOS SECTOR
		POP	AX
		MOV	DL,AL		; DL = DISK
.L0:		MOV	AX,0X0201	; READ 1 SECTOR
		INT	0X13
		JC	.L0		; ON ERROR, TRY AGAIN

		POP	BX
		POP	CX
		POP	DX
		POP	AX

;		INC	AX		wtf.. this doesn't work.
		ADD	AX,1
		ADC	DX,0		; INC DX:AX
		ADD	BX,0X0200	; ES:BX += 0X200

		DEC	DI		; WHILE (--DI!=0)
		JNZ	READ_SECTORS
		RET

LONG_TO_BIOS:	; INPUT- DX:AX = SECTOR
		PUSH	AX
		PUSH	DX

		MOV	AL,BYTE[0X7C00+BOOT_HEADS]
		XOR	AH,AH		; AX = HEADS
		MOV	DL,BYTE[0X7C00+BOOT_SECTORS]
		XOR	DH,DH		; DX = SECTORS
		MUL	DX		; AX = HEADS*SECTORS
		XCHG	AX,CX		; CX = AX (HEADS*SECTORS)

		POP	DX		; DX:AX = SECTOR NR
		POP	AX

		DIV	CX		; AX = TRACK, DX = SECTOR/HEAD
		PUSH	AX		; SAVE TRACK NR

		XOR	AX,AX
		XCHG	AX,DX		; DX:AX = SECTOR/HEAD
		MOV	CL,BYTE[0X7C00+BOOT_SECTORS]
		XOR	CH,CH		; CX = SECTORS PER TRACK

		DIV	CX		; DX = SECTOR, AX = HEAD
		XCHG	AX,DX
		INC	AX		; SECTOR++

		POP	CX		; CX = TRACK NR
		CMP	BYTE[0X7C00+BOOT_DISK],0X80
		JB	.FD		; IF FLOPPY, JMP TO .FD

		SHL	CX,6		; CX <<= 6
		OR	CL,AL		; CX += DL (CX = TRACK/SECTOR)
		JMP	SHORT .HD
.FD:		MOV	CH,CL		; CH = TRACK
		MOV	CL,AL		; CL = SECTOR (CX = TRACK/SECTOR)
.HD:
		MOV	DH,DL		; DH = HEAD

		RET


; *** DATA SECTION ***

KERNEL_ENTRY	DD	0

; 0000		02	Segment length (low 16 bits)
; 0002		02	Base address (low 16 bits)
; 0004		01	Base address (bits 16-23)
; 0005		01	Flags
; 0006		01	Access + segment length (bits 16-19)
; 0007		01	Base address (bits 24-31)

; Flags:

; 0000		04	Type (3 = Data, 11 = Code)
; 0004		01	DT (1 if data or code, 0 is system)
; 0005		02	DPL (level... should be 0)
; 0007		01	P (1 if valid, 0 if invalid)

; Access byte

; 0000		04	Segment length (bits 0-3)
; 0004		01	AVL (0)
; 0005		01	0
; 0006		01	Default operand size (1 = 32-bit)
; 0007		01	Granularity (1 = 4K, 0 = 1 byte)


GDT_TABLE:
		DW	GDT_SIZE
		DD	DUMMY
		DW	0

DUMMY:		DW	0,0,0,0

CODE_32:	DW	0XFFFF
		DW	0X0000
		DB	0X00
		DB	0X9A
		DB	0XCF
		DB	0X00

DATA_32:	DW	0XFFFF
		DW	0X0000
		DB	0X00
		DB	0X92
		DB	0XCF
		DB	0X00

GDT_SIZE	EQU	$-DUMMY-1

%IF LANGUAGE = ENGLISH
STR_LOADING_KERNEL	DB	"Searching for kernel...",0
STR_KERNEL_FOUND	DB	"found.",0X0D,0X0A,0
STR_NO_KERNEL		DB	"not found.",0X0D,0X0A
			DB	"Press CTRL+ALT+DEL to reboot.",0
STR_KERNEL_LOADED	DB	"Kernel loaded.",0X0D,0X0A,0
STR_SECTOR_TABLE_LOADED	DB	"Sector table loaded.",0X0D,0X0A,0
%ENDIF

%IF LANGUAGE = SWEDISH
STR_LOADING_KERNEL	DB	"S",148,"ker efter kernel...",0
STR_KERNEL_FOUND	DB	"hittad.",0X0D,0X0A,0
STR_NO_KERNEL		DB	"ej hittad.",0X0D,0X0A
			DB	"Tryck p",134," CTRL+ALT+DEL  f",148,"r"
			DB	" att starta om datorn.",0
STR_KERNEL_LOADED	DB	"Kernel laddad.",0X0D,0X0A,0
STR_SECTOR_TABLE_LOADED	DB	"Sektortabell laddad.",0X0D,0X0A,0
%ENDIF

%IF LANGUAGE = GERMAN
STR_LOADING_KERNEL 	DB      "Suche nach Kernel...",0
STR_KERNEL_FOUND       	DB      "gefunden.",0X0D,0X0A,0
STR_NO_KERNEL		DB	"nicht gefunden.",0X0D,0X0A
                        DB	"Druecken Sie CTRL+ALT+DEL zum Reboot.",0
STR_KERNEL_LOADED       DB	"Kernel geladen.",0X0D,0X0A,0
STR_SECTOR_TABLE_LOADED	DB	"Sektortabelle geladen.",0X0D,0X0A,0
%ENDIF

%IF LANGUAGE = FRENCH
STR_LOADING_KERNEL      DB      "Recherche du noyau...",0
STR_KERNEL_FOUND        DB      "Trouv.",0X0D,0X0A,0
STR_NO_KERNEL           DB      "Introuvable...",0X0D,0X0A
                        DB      "Appuyez sur CTRL+ALT+DEL pour redmarrer l'ordinateur.",0
STR_KERNEL_LOADED       DB      "Noyau charg.",0X0D,0X0A,0
STR_SECTOR_TABLE_LOADED DB      "Table de secteurs charge.",0X0D,0X0A,0
%ENDIF

KERNEL_SIZE		DD	0
KERNEL_NAME		DB	KERNEL_FILENAME,0
HEXTABLE		DB	"0123456789ABCDEF"

%INCLUDE	'bootsector.inc'
%INCLUDE	'filetable.inc'
%INCLUDE	'sectortable.inc'
