/*
	*** IMPORTANT ***
	
	The structures in this file needs to be compiled with -fpack-struct, some people says this may case errors. Be careful.
	(2001-09-03 Robert stling)
*/

#define	SECTOR_FREE		0xffffffff
#define	SECTOR_BAD		0xfffffffe
#define	SECTOR_SYSTEM	0xfffffffd
#define	SECTOR_LAST		0xfffffffc
#define	SECTOR_SPECIAL	0xfffffff8	// (ver 0.2: No sector numbers over SECTOR_SPECIAL are real sector numbers)

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;

typedef struct {
	word	Jump;				// 0000	jmp short code 0x2eeb
	word	BozID;				// 0002	0x4191 = 0x91 0x41
	dword	BaseSector;			// 0004	Base sector of partition, all other sector nrs are relative to this
	dword	Size;				// 0008	Size in sectors
	byte	Heads;				// 000c	Heads on disk
	word	Tracks;				// 000d	Tracks per head
	byte	Sectors;			// 000f	Sectors per track
	byte	Disk;				// 0010	Disk nr. 0 = fd0, 0x80 = hd0, 0x81 = hd1 etc.
	dword	BootLoader;			// 0011	Boot loader sector
	byte	BootLoaderSize;		// 0015	Size of boot loader
	dword	SectorTable;		// 0016	Sector allocation table sector
	dword	FileTable;			// 001a	Root directory sector
	byte	FileTableSize;		// 001e	Root directory size (sectors)
	byte	Reserved[0x11];		// 001f	Reserved bytes
	byte	Code[0x1ce];		// 0030	Code to load the boot loader, hehe. Boot loader loader :-) 
	word	Magic;				// 01fe 0xaa55
} TBootSector;
// 512 (0x200) bytes

typedef struct {
	dword	Size;				// 0000	Size in bytes
	dword	Sector;				// 0004	Start sector
	dword	Attrib;				// 0008	Attributes (ver 0.2: changed from word to dword)
	word	User;				// 000c	UID of owner
	word	Group;				// 000e GID of group
	char	Name[0x28];			// 0010	0x28 (40) char name, zero-terminated (so max length is 39)
	byte	Reserved[0x8];		// 0038	0x08 reserverd bytes
} TFileEntry;
// 64 (0x40) bytes

/* 3.5" 1440KB */
TBootSector bsfd1440 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	2880,						// Size in sectors
	2,							// Heads
	80,							// Tracks/head
	18,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* Robert stling, 2001-10-17: Added 6 more floppy formats */

/* 3.5" 720KB */
TBootSector bsfd720 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	1440,						// Size in sectors
	2,							// Heads
	80,							// Tracks/head
	9,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* 5.25" 1.2MB */
TBootSector bsfd1200 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	2400,						// Size in sectors
	2,							// Heads
	15,							// Tracks/head
	9,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* 5.25" 360KB */
TBootSector bsfd360 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	720,						// Size in sectors
	2,							// Heads
	40,							// Tracks/head
	9,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* 5.25" 180KB */
TBootSector bsfd180 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	360,						// Size in sectors
	1,							// Heads
	40,							// Tracks/head
	9,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* 5.25" 320KB */
TBootSector bsfd320 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	640,						// Size in sectors
	2,							// Heads
	40,							// Tracks/head
	8,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* 5.25" 160KB */
TBootSector bsfd160 = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	320,						// Size in sectors
	1,							// Heads
	40,							// Tracks/head
	8,							// Sectors/track
	0,							// Disk (fd0)
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	28,							// File table
	8							// File table size (4KB, 64 files)
};

/* Robert stling 2001-12-07: Added hd */

TBootSector bshd = {
	0x2eeb,						// Jump code
	0x4191,						// BozFS ID
	0,							// Base sector
	0,							// Size in sectors
	0,							// Heads
	0,							// Tracks/head
	0,							// Sectors/track
	0,							// Disk
	1,							// Boot loader sector
	2,							// Boot loader size (1KB)
	5,							// Sector table
	5,							// File table
	8							// File table size (4KB, 64 files)
};
