; charfile.asm
;
; Robert stling, 2002-05-10
;
; "Files", only used for charachter devices.

%define _charfile_asm 0

[bits 32]
[section .text]

%include "charfile.inc"
%include "chardev.inc"
%include "mem.inc"
%include "llist.inc"
%include "string.inc"


global charfile_open
global charfile_close
global charfile_read_char
global charfile_write_char
global charfile_read
global charfile_write

global charfile_set_blocking
global charfile_get_driver_version
global charfile_check_for_data
global charfile_switch_device


; charfile_get_driver_device
;
; esi = name
;
; Returns:
; eax = driver pointer (0 if error)
; ebx = device

charfile_get_driver_device:
		push	ecx
		push	edx
		push	esi
		push	edi

		mov	edi,charfile_drivername
.nextchar:
		lodsb
		cmp	al,'('
		jz	.get_number
		or	al,al
		jz	.error
		stosb
		jmp	.nextchar
.get_number:
		mov	al,0
		stosb
		xor	edi,edi
.nextnumber:
		call	string_dec_to_int
		movzx	ebx,bl
		shl	edi,8
		or	edi,ebx
		inc	esi
		cmp	byte[esi-1],','
		jz	.nextnumber
		cmp	byte[esi-1],')'
		jnz	.error
		cmp	byte[esi],0
		jnz	.error

		mov	esi,charfile_drivername
		call	chardev_get_pointer_from_name
		or	eax,eax
		jz	.error

		mov	ebx,edi

		jmp	short .quit
.error:
		xor	eax,eax
.quit:
		pop	edi
		pop	esi
		pop	edx
		pop	ecx
		ret



; charfile_open
;
; esi = "file" name
;
; Returns:
; eax = 0 - OK, 1 - error, 2 - device is busy
; esi = file struct pointer

charfile_open:
		push	ecx
		push	edx
		push	ebx



		call	charfile_get_driver_device
		or	eax,eax
		jz	near .error

		mov	ecx,dword[charfile_list]
		mov	edx,ecx
.try_next:
		or	ecx,ecx
		jz	.not_found

		mov	edx,ecx
		mov	ecx,dword[edx+charfile_desc.next]

		cmp	dword[edx+charfile_desc.driver],eax
		jnz	.try_next
		cmp	dword[edx+charfile_desc.device],ebx
		jnz	.try_next

		mov	eax,2
		jmp	short .quit
.not_found:

		push	eax
		push	ebx

		dllist_create charfile_desc, charfile_list

		pop	ebx
		pop	eax

		mov	dword[esi+charfile_desc.driver],eax
		mov	dword[esi+charfile_desc.device],ebx
		xor	eax,eax
		jmp	short .quit
.error:
		mov	eax,1
.quit:
		pop	ebx
		pop	edx
		pop	ecx
		ret



; charfile_close
;
; eax = file struct pointer
;
; Returns:
; All register unmodified

charfile_close:
		push	eax
		dllist_delete charfile_desc,charfile_list,eax
		pop	eax
		ret



; charfile_read_char
;
; eax = file struct pointer
;
; Returns:
; ecx = data
; edx = 0 if data is read, 1 if no data is avaiable

charfile_read_char:
		push	eax

		mov	ecx,dword[eax+charfile_desc.device]
		mov	eax,dword[eax+charfile_desc.driver]
		chardev_call eax,chardev_read

		pop	eax
		ret



; charfile_read
;
; Device must be in blocking mode!
;
; eax = file struct pointer
; ecx = size
; edi = destination buffer
;
; Returns:
; eax = 0 if OK

charfile_read:
		push	ecx
		push	esi
		push	edi

		mov	esi,eax
.read_next:
		push	ecx

		mov	ecx,dword[esi+charfile_desc.device]
		mov	eax,dword[esi+charfile_desc.driver]
		chardev_call eax,chardev_read

		mov	byte[edi],cl
		inc	edi

		pop	ecx

		or	edx,edx
		jnz	.error

		loop	.read_next

		xor	eax,eax
		jmp	short .quit
.error:
		mov	eax,1
.quit:
		pop	edi
		pop	esi
		pop	ecx
		ret



; charfile_write_char
;
; eax = file struct pointer
; ecx = charachter to print
;
; Returns:
; eax = 0 if OK, else eax != 0

charfile_write_char:
		push	ecx
		push	edx

		mov	edx,ecx
		mov	ecx,dword[eax+charfile_desc.device]
		mov	eax,dword[eax+charfile_desc.driver]
		chardev_call eax,chardev_write

		mov	eax,edx

		pop	edx
		pop	ecx
		ret



; charfile_write
;
; eax = file struct pointer
; ecx = size
; esi = buffer
;
; Returns:
; eax = 0 if OK


charfile_write:
		push	ecx
		push	edx
		push	ebx
		push	esi

		mov	ebx,eax
.next:
		push	ecx

		lodsb
		xchg	eax,edx
		mov	ecx,dword[ebx+charfile_desc.device]
		mov	eax,dword[ebx+charfile_desc.driver]
		chardev_call eax,chardev_write

		pop	ecx
		loop	.next

		xor	eax,eax

		pop	esi
		pop	ebx
		pop	edx
		pop	ecx
		ret



; charfile_set_blocking
;
; Set blocking or non-blocking mode.
; This affects the whole driver! Not just the device.
;
; eax = file struct pointer
; cl = mode to set (0 = off, 1 = on)

charfile_set_blocking:
		push	eax
		push	ecx

		mov	ch,cl
		mov	cl,0x10
		mov	eax,dword[eax+charfile_desc.driver]
		chardev_call eax,chardev_options

		pop	ecx
		pop	eax
		ret



; charfile_get_driver_version
;
; eax = file struct pointer
; Returns:
; ch = major version, cl = minor version

charfile_get_driver_version:
		push	eax

		xor	cl,cl
		mov	eax,dword[eax+charfile_desc.driver]
		chardev_call eax,chardev_options

		pop	eax
		ret




; charfile_check_for_data
;
; eax = file struct pointer
; Returns:
; cl = 0 - no data, cl = 1 - data pending

charfile_check_for_data:
		push	eax
		push	edx

		mov	cl,0x01
		mov	edx,dword[eax+charfile_desc.device]
		mov	eax,dword[eax+charfile_desc.driver]
		chardev_call eax,chardev_options

		pop	edx
		pop	eax
		ret



; charfile_switch_device
;
; eax = file struct pointer
; ch = device
; Returns:
; All registers unchanged.

charfile_switch_device:
		push	eax
		push	ecx

		mov	cl,0x11
		mov	eax,dword[eax+charfile_desc.driver]
		chardev_call eax,chardev_options

		pop	ecx
		pop	eax
		ret



[section .data]

charfile_list		dd	0


[section .bss]

charfile_drivername	resb	0x100
