; console.asm
;
; Robert stling, 2002-02-07
;
; Virtual consoles.


%define _console_asm 0

%include "console.inc"
%include "crt.inc"
 %include "mem.inc"

[bits 32]
[section .text]

global	console_allocate
global	console_switch_to
global	console_print_string
global	console_clear
global	console_print_char
global	console_set_cursor
global	console_get_char_pos
global	console_set_char_pos
global	console_set_attrib
global	console_move_cursor_forward
global	console_move_cursor_back
global	console_move_cursor_down
global	console_move_cursor_up
global	console_get_pointer


; console_allocate
;
; Allocates eax numbers of virtual consoles, clears them, and sets number 0 to active.
;
; eax = number of virtual consoles
;
; returns:
; eax = 0 if error, else 1

console_allocate:
		push	ecx
		push	ebx
		push	esi
		push	edi

		mov	dword[console_count],eax	; save the number of consoles
		mov	ecx,eax
		shl	eax,2				; eax *= 4 (size of console table)
		call	mem_alloc
		or	esi,esi				; if((esi=malloc())==NULL) return 1;
		jnz	.mem_ok

.error:
		mov	eax,1
		jmp	short .quit

.mem_ok:

		mov	dword[console_list],esi
		mov	edi,esi
.allocate_next_console:
		mov	eax,screen_size			; allocate a new screen structure
		call	mem_alloc
		or	esi,esi
		jz	.error				; return 1 if error...

		mov	ebx,esi
		call	crt_create_screen		; create the screen (allocate buffer, initialize the screen struct)
		or	eax,eax
		jnz	.error

		call	crt_clear_screen

		mov	dword[edi+(ecx*4)-4],esi	; move the pointer to this screen struct to the entry
		loop	.allocate_next_console		; in the console_list

		mov	dword[console_active],esi
		mov	byte[esi+screen.active],1
		mov	ebx,esi
		call	crt_load_screen
		call	crt_set_cursor_physical

		xor	eax,eax
.quit:
		pop	edi
		pop	esi
		pop	ebx
		pop	ecx

		ret


; console_switch_to
;
; Allocates eax numbers of virtual consoles, clears them, and sets number 0 to active.
;
; ebx = console number to switch to
;
; returns:
; eax = 0 if ok, else 1

console_switch_to:
		push	ecx
		push	edx
		push	ebx

		cmp	ebx,dword[console_count]
		jb	.ok

		mov	eax,1
		jmp	short .done

.ok:
		mov	edx,ebx
		mov	ecx,dword[console_list]	;	 load a pointer to the console_list
		mov	ebx,dword[ecx+(ebx*4)]		; ebx is now the ebx:th entry in the table
		cmp	ebx,dword[console_active]
		jz	.done				; heh, idiot, you can't change to the current screen

		mov	ebx,dword[console_active]	; ebx = currently active screen

		call	crt_save_screen			; save screen to memory buffer, and mark it as inactive
		mov	byte[ebx+screen.active],0

		mov	ecx,[console_list]		; edx = console number to set active
		mov	ebx,dword[ecx+(edx*4)]		; ebx = console_list[edx]
		mov	dword[console_active],ebx
		mov	byte[ebx+screen.active],1

		call	crt_load_screen
		call	crt_set_cursor_physical

		xor	eax,eax
.done:
		pop	ebx
		pop	edx
		pop	ecx
		ret



; console_clear
;
; clear the console
;
; ebx = console number
;
; returns:
; eax = 0 if ok, else 1

console_clear:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:

		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_clear_screen		; clear that console
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret



; console_print_string
;
; prints a string to the console
;
; ebx = console number
; esi = string to print
;
; returns:
; eax = 0 if ok, else 1

console_print_string:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_put_string			; print string to that console
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret



; console_print_char
;
; prints a char to the console
;
; ebx = console number
; dl = char to print
;
; returns:
; eax = 0 if ok, else 1

console_print_char:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_put_char			; print a char to that console
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret



; console_set_cursor
;
; sets the cursor position
;
; ebx = console number
; cl = x
; ch = y
;
; returns:
; eax = 0 if ok, else 1

console_set_cursor:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_set_cursor
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret



; console_get_char_pos
;
; get charachter at position
;
; ebx = console number
; cl = x
; ch = y
;
; returns:
; eax = 0 if ok, else 1
; dl = char

console_get_char_pos:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_get_char_pos
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret



; console_set_char_pos
;
; set charachter at position
;
; ebx = console number
; cl = x
; ch = y
; dl = char
;
; returns:
; eax = 0 if ok, else 1

console_set_char_pos:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_set_char_pos
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret



; console_set_attrib
;
; set the console default attribute
;
; ebx = console number
; dl = attribute
;
; returns:
; eax = 0 if ok, else 1

console_set_attrib:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		mov	byte[ebx+screen.attrib],dl
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret


; console_move_cursor_forward
;
; moves the cursor one step forward
;
; ebx = console number
;
; returns:
; eax = 0 if ok, else 1

console_move_cursor_forward:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_move_cursor_forward
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret




; console_move_cursor_back
;
; moves the cursor one step back
;
; ebx = console number
;
; returns:
; eax = 0 if ok, else 1

console_move_cursor_back:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_move_cursor_back
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret




; console_move_cursor_down
;
; moves the cursor one step down
;
; ebx = console number
;
; returns:
; eax = 0 if ok, else 1

console_move_cursor_down:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_move_cursor_down
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret




; console_move_cursor_up
;
; moves the cursor one step up
;
; ebx = console number
;
; returns:
; eax = 0 if ok, else 1

console_move_cursor_up:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		mov	eax,1
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	ebx,dword[eax+(ebx*4)]		; ebx = console_list[ebx]
		call	crt_move_cursor_up
		xor	eax,eax				; return 0
.done:
		pop	ebx
		ret


; console_get_pointer
;
; ebx = console number
;
; returns:
; eax = pointer to screen struct, 0 if error

console_get_pointer:
		push	ebx

		cmp	ebx,dword[console_count]	; check if the console number is valid
		jb	.ok				; if not, return 1

		xor	eax,eax
		jmp	short .done
.ok:
		mov	eax,dword[console_list]
		mov	eax,dword[eax+(ebx*4)]	;	 eax = console_list[ebx]
.done:
		pop	ebx
		ret

[section .bss]

console_count	resd	1
console_list	resd	1
console_active	resd	1
