; gdt.asm
;
; Robert Ostling, 2002-01-27
;
;
; This contains a function to set up the GDT. We only need 2 segments, one code
; segment, and one data segment, both 4GB in size.
;
;
; GDT entry
;
; segment length (bits 0-15)	word
; base address (bits 0-15)	word
; base address (bits 16-23)	byte
; flags				byte
; access and segment length	byte
; base address (bits 24-31)	byte
;
; flags
;
; type				bits 0-3
;  (examples)
;  0x02 = r/w data
;  0x0a = exec/read code
; descriptor type		bit 4
;  0 = system segment/gate
;  1 = data/code
; privilege level		bits 5-6
;  0 = highest, 3 = lowest
; presence			bit 7
;  0 = not present
;  1 = present
;
;
; access byte
;
; segment length (bits 16-23)	bits 0-3
; 0				bits 4-5
; operand size			bit 6
;  0 = 16-bit
;  1 = 32-bit
; granularity			bit 7
;  0 = 1 byte
;  1 = 4K bytes
;
;

[bits 32]
[section .text]

%define _gdt_asm 0

global gdt_init

; gdt_init
;
; loads the GDT
; no registers modified

gdt_init:
		lgdt	[gdt_descriptor]
		mov	ax,0x0010
		mov	ds,ax
		mov	es,ax
		mov	gs,ax
		mov	fs,ax
		mov	ss,ax
		jmp	0x0008:.new_cs
.new_cs:
		ret

[section .data]

gdt:
.dummy		times	8 db 0
.codeseg:
		dw	0xffff
		dw	0x0000
		db	0x00
		db	0x9a
		db	0xcf
		db	0x00
.dataseg:
		dw	0xffff
		dw	0x0000
		db	0x00
		db	0x92
		db	0xcf
		db	0x00

.end:

gdt_descriptor:
.size		dw	(gdt.end - gdt) - 1
.pointer	dd	gdt
