; idt.asm
;
; Robert Ostling, 2002-01-27
;
;
; This contains functions to set and modify up the IDT.
;
;
; IDT entry
;
; address (0-15)	word
; selector		word
; flags			word
; address (16-31)	word
;
; flags
;
; 0x0e00		bits 0-12
; privilege level	bits 13-14
;  0 = highest
;  ...
;  3 = lowest
; presence		bit 15
;  0 = not present
;  1 = present
;
;

%define _idt_asm 0

[bits 32]
[section .text]

global idt_init
global idt_set_interrupt_vector

%include "idt.inc"


; idt_init
;
; initializes the IDT.
; all registers unchanged.

idt_init:
		lidt	[idt_descriptor]
		ret


; idt_set_interrupt_vector
;
; eax = interrupt number
; ebx = pointer to interrupt function

idt_set_interrupt_vector:
		push	ebx

		mov	word[idt + (eax*8)],bx
		shr	ebx,16
		mov	word[idt + (eax*8) + 6],bx
						; set flags + selector (code segment)
		mov	dword[idt + (eax*8) + 2],0x8e000008

		pop	ebx
		ret


[section .data]

idt_descriptor:
.size		dw	0x7ff			; (0x100 interrupts * 8 bytes) - 1
.pointer	dd	idt


[section .bss]

idt		resb	0x800
