; ipc.asm
;
; Robert stling, 2002-03-02
;
; Interprocess communication.

%define _ipc_asm 0

[bits 32]
[section .text]


%include "ipc.inc"
%include "proc.inc"
%include "mem.inc"


global	ipc_send_message
global	ipc_check_message
global	ipc_receive_message


; ipc_send_message
;
; eax = PID
; ecx = parameter 0
; edx = parameter 1
; ebx = parameter 2
; esi = parameter 3
;
; returns:
; eax = 0 if ok, 1 if error


ipc_send_message:
		cli
		push	ecx
		push	edx
		push	ebx
		push	esi

		call	proc_get_pointer	; get pointer to proc_struct
		or	eax,eax			; error?
		jnz	.ok			; nope..

		mov	eax,1
		jmp	.quit
.ok:
		mov	dword[ipc_tmpdata],ecx
		mov	dword[ipc_tmpdata+4],edx
		mov	dword[ipc_tmpdata+8],ebx
		mov	dword[ipc_tmpdata+12],esi

		mov	ebx,dword[eax+proc_struct.info]	; ebx = pointer to proc_info
		mov	ecx,dword[ebx+proc_info.ipc_list]
		mov	edx,ecx			; edx = ecx = pointer to ipc_list
.find_next:
		or	ecx,ecx			; have we found the last entry?
		jz	.found

		mov	edx,ecx			; edx = old pointer to proc_info
		mov	ecx,dword[ecx+ipc_struct.next]
		jmp	short .find_next
.found:
		mov	eax,ipc_struct_size	; edx now points to the last element in the list
		call	mem_alloc		; allocate a new ipc_struct

		or	esi,esi			; did it return NULL?
		jnz	.mem_ok			; nah.. everything's OK

		mov	eax,1			; yes, return 1
		jmp	short .quit
.mem_ok:

		or	edx,edx			; is this new struct the first in the list?
		jz	.first			; yup, skip the following
		mov	dword[edx+ipc_struct.next],esi	; make the old element point to the new one
		jmp	short .not_first
.first:						; now let's set this new element as the first (and only)
		mov	dword[ebx+proc_info.ipc_list],esi
.not_first:
		mov	dword[esi+ipc_struct.next],0	; the new task has nothing in front of it

		mov	eax,dword[ipc_tmpdata]	; copy the data
		mov	dword[esi+ipc_struct.data],eax
		mov	eax,dword[ipc_tmpdata+4]
		mov	dword[esi+ipc_struct.data+4],eax
		mov	eax,dword[ipc_tmpdata+8]
		mov	dword[esi+ipc_struct.data+8],eax
		mov	eax,dword[ipc_tmpdata+12]
		mov	dword[esi+ipc_struct.data+12],eax

		call	proc_get_current_pid
		mov	dword[esi+ipc_struct.sender],eax
		xor	eax,eax			; return 0

.quit:
		pop	esi
		pop	ebx
		pop	edx
		pop	ecx
		sti
		ret



; ipc_check_message
;
; returns:
; eax = pointer to ipc_struct, 0 if nothing avaiable

ipc_check_message:
		mov	eax,dword[proc_current]
		mov	eax,dword[eax+proc_struct.info]
		mov	eax,dword[eax+proc_info.ipc_list]
		ret



; ipc_receive_message
;
; returns:
; eax = status (0 = OK)
; ecx = parameter 0
; edx = parameter 1
; ebx = parameter 2
; esi = parameter 3
; edi = sender PID


ipc_receive_message:
		push	ebp
.wait:

		call	ipc_check_message	; get address to message
		or	eax,eax			; is it NULL?
		jz	.wait			; yes, wait until a message has arrived

		mov	ecx,eax
		call	mem_free
		or	eax,eax
		jz	.ok

		mov	eax,1
		jmp	.quit
.ok:
		cli

		xchg	eax,ecx			; eax points to ipc_struct
		mov	ecx,dword[eax+ipc_struct.data]
		mov	edx,dword[eax+ipc_struct.data+4]
		mov	ebx,dword[eax+ipc_struct.data+8]
		mov	esi,dword[eax+ipc_struct.data+12]
		mov	edi,dword[eax+ipc_struct.sender]
		mov	ebp,dword[eax+ipc_struct.next]

		sti

		mov	eax,dword[proc_current]
		mov	eax,dword[eax+proc_struct.info]
		mov	dword[eax+proc_info.ipc_list],ebp

		xor	eax,eax
.quit:
		pop	ebp
		ret


[section .bss]

ipc_tmpdata	resd	4
