; irq.asm
;
; Robert Ostling, 2002-01-27
;
; Used to enable, disable IRQs, and to remap them to interrupt vectors 0x20-0x2f

%define _irq_asm 0

[bits 32]
[section .text]

global	irq_set_mask
global	irq_get_mask
global	irq_enable
global	irq_disable

global	irq_remap

%include "irq.inc"

; irq_enable
;
; enable an IRQ, using either the master or slave PIC
;
; cl = irq

irq_enable:
		push	eax
		push	ecx

		cmp	cl,8
		jb	.master

		sub	cl,8
		mov	ah,1
		shl	ah,cl
		xor	ah,0xff

		in	al,0xa1
		and	al,ah
		out	0xa1,al

		pop	ecx
		pop	eax
		ret
.master:
		mov	ah,1
		shl	ah,cl
		xor	ah,0xff

		in	al,0x21
		and	al,ah
		out	0x21,al

		pop	ecx
		pop	eax

		ret


; irq_disable
;
; disable an IRQ, using either the master or slave PIC
;
; cl = irq

irq_disable:
		push	eax
		push	ecx

		cmp	cl,8
		jb	.master

		sub	cl,8
		mov	ah,1
		shl	ah,cl

		in	al,0xa1
		or	al,ah
		out	0xa1,al

		pop	ecx
		pop	eax
		ret
.master:
		mov	ah,1
		shl	ah,cl

		in	al,0x21
		or	al,ah
		out	0x21,al

		pop	ecx
		pop	eax
		ret


; irq_get_mask
;
; returns the PIC mask for both slave and master controller
;
; returns:
; ax = bit map (IRQn is bit n)

irq_get_mask:
		in	al,0xa1
		mov	ah,al
		in	al,0x21
		ret


; irq_set_mask
;
; sets the PIC mask for both slave and master controller
;
; ax = bit map (IRQn is bit n)
;
; all registers unchanged

irq_set_mask:
		out	0x21,al
		mov	al,ah
		out	0xa1,al
		ret


; irq_remap
;
; this remaps IRQ 0-7 to interrupt 0x20 - 0x27,
; and IRQ 8-15 (slave PIC) to 0x28-0x2f

irq_remap:
		push	eax

		mov	al,0x11
		out	0x20,al

		mov	al,0x20
		out	0x21,al
		mov	al,0x04
		out	0x21,al
		mov	al,0x01
		out	0x21,al
		mov	al,0xff
		out	0x21,al

		mov	al,0x11
		out	0xa0,al

		mov	al,0x28
		out	0xa1,al
		mov	al,0x02
		out	0xa1,al
		mov	al,0x01
		out	0xa1,al
		mov	al,0xff
		out	0xa1,al

		pop	eax
		ret
