; kernel.asm
;
; Robert stling, 2002-01-27
;
;
; This file contains the kernel entry point
;
; 2002-02-02:
; Removed multiboot header, converted to EBFS (Extended BozOS File System)
;
; 2002-02-03:
; Moved some things to .inc files
;
; 2002-05-12:
; Heh... I've done quite a lot the last months ;)
;
; 2002-05-12:
; Haha! The bootloader didn't initialize ESP, and THAT caused trouble :)


[bits 32]
[section .text]

global	_start

%include "config.inc"

%include "gdt.inc"
%include "idt.inc"
%include "mem.inc"
%include "irq.inc"
%include "pit.inc"
%include "timer.inc"
%include "proc.inc"
%include "crt.inc"
%include "kbd.inc"
%include "parallel.inc"
%include "ipc.inc"
%include "serial.inc"
%include "chardev.inc"
%include "pipe.inc"
%include "screen.inc"
%include "string.inc"
%include "charfile.inc"


_start:

		cli				; Clear interrupts, I feel safer this way.

		call	gdt_init		; Load our GDT, and set cs/ds/es/fs/gs/ss.
		call	idt_init		; Load our IDT.

		mov	ax,0xffff
		call	irq_set_mask		; Disable all IRQs.
		call	irq_remap		; Remap all IRQs to interrupt 0x20 - 0x2f.
		sti				; All IRQs disables, let's enable interrupts again.

		call	mem_init		; Initialize memory manager.
		call	pit_init		; Initialize PIT (timer)

		mov	eax,0x4000		; 16 KB
		call	mem_alloc		; Allocate stack space for main kernel thread

		mov	esp,esi
		add	esp,0x4000

		mov	edx,esi
		call	proc_init		; Start multitasking

						; Now initialize the charachter device drivers.
		call	screen_chardev_init	; screen (this has caused problem when not been initialized first)
						; make sure to examine this later.
		call	serial_chardev_init	; serial port
		call	kbd_chardev_init	; keyboard

		mov	esi,kernel_default_stdin
		call	charfile_open		; open stdin
		or	eax,eax
		jnz	.error_stdin		; error...

		mov	ebx,esi			; save stdin pointer in ebx

		mov	esi,kernel_default_stdout
		call	charfile_open		; open stdout
		or	eax,eax
		jnz	.error_stdin		; error...

		call	proc_get_current_info	; eax now points to our proc_info struct :)

		mov	dword[eax+proc_info.stdin],ebx
		mov	dword[eax+proc_info.stdout],esi

		call	proc_get_current_stdout
		mov	ecx,kernel_welcome_size
		mov	esi,kernel_welcome
		call	charfile_write

.error_stdin:
.error_stdout:
		call	parallel_init
		xor	ecx,ecx
		mov	al,0x06
		call	parallel_send

.infinite_loop:
		jmp	short .infinite_loop



[section .data]

kernel_default_stdin	db	"kbd(0)",0
kernel_default_stdout	db	"screen(0)",0

kernel_welcome:
		db	"Welcome to Primula!",0x0a,0x0a
		db	"Note: you can not do anything with this version. Soon I will release",0x0a
		db	"something funny for you to play with, check http://ostling.no-ip.com/",0x0a
		db	"for the latest information, or come to #primula on irc.openprojects.net",0x0a
		db	"You could also mail me at robost@telia.com.",0x0a,0x0a
		db	"Press left shift + right shift to reboot.",0x0a

kernel_welcome_size	equ	$-kernel_welcome

[section .bss]
