; llist.inc
;
; Macros for linked lists, Robert stling 2002-05-02
;
; 2002-05-12: Fixed a NASTY bug in dllist_delete... Bleh...


; dllist_create struc list
;
; "struc" is the structure to use.
; "list" is the address of a dword where a pointer to the first element in
; the list is stored (or 0 is the list is empty).
;
; Returns:
; eax = 0 if everything's OK
; esi = pointer to the new element

%macro dllist_create 2

		push	edx

		mov	eax,dword[%2]			; eax points to the current element
		mov	edx,eax				; edx points to last element
%%not_null:
		or	eax,eax				; have we found the last element of the list?
		jz	%%is_null			; yeah
		mov	edx,eax				; nope, last element = current element
		mov	eax,dword[eax+%1.next]
		jmp	short %%not_null		; check the next element
%%is_null:
		mov	eax,%1_size			; allocate memory for the new element
		call	mem_alloc
		or	esi,esi				; did we fail?
		jnz	%%mem_ok			; nope

		mov	eax,1				; yeah, return 1
		jmp	%%quit
%%mem_ok:
		or	edx,edx				; was there any previous element?
		jz	%%first				; nope, jump.
		mov	dword[edx+%1.next],esi
		jmp	short %%not_first		; yes, set old.next to new
%%first:
		mov	dword[%2],esi			; no, make this the first element in the list
%%not_first:
		mov	dword[esi+%1.next],0		; this is the last element of the list
		mov	dword[esi+%1.prev],edx

		xor	eax,eax				; return 0
%%quit:
		pop	edx

%endmacro



; dllist_delete struc list element
;
; "struc" is the structure to list
; "list" is the address of a dword where a pointer to the first element in the list is stored
; "element" is the address of the element to delete
;
; Returns:
; eax = 0 is everything's OK

%macro dllist_delete 3

		push	ecx
		push	edx

		mov	eax,%3
		mov	ecx,dword[eax+%1.next]
		mov	edx,dword[eax+%1.prev]
		or	ecx,ecx
		jz	%%no_element_after

		mov	dword[ecx+%1.prev],edx
%%no_element_after:
		or	edx,edx
		jz	%%no_element_before

		mov	dword[edx+%1.next],ecx
		jmp	short %%element_before
%%no_element_before:
		mov	dword[%2],ecx
%%element_before:
		call	mem_free

		pop	edx
		pop	ecx

%endmacro




; sllist_create struc list
;
; "struc" is the structure to use.
; "list" is the address of a dword where a pointer to the first element in
; the list is stored (or 0 is the list is empty).
;
; Returns:
; eax = 0 if everything's OK
; esi = pointer to the new element

%macro sllist_create 2

		push	edx

		mov	eax,dword[%2]			; eax points to the current element
		mov	edx,eax				; edx points to last element
%%not_null:
		or	eax,eax				; have we found the last element of the list?
		jz	%%is_null			; yeah
		mov	edx,eax				; nope, last element = current element
		mov	eax,dword[eax+%1.next]
		jmp	short %%not_null		; check the next element
%%is_null:
		mov	eax,%1_size			; allocate memory for the new element
		call	mem_alloc
		or	esi,esi				; did we fail?
		jnz	%%mem_ok			; nope

		mov	eax,1				; yeah, return 1
		jmp	%%quit
%%mem_ok:
		or	edx,edx				; was there any previous element?
		jz	%%first				; nope, jump.
		mov	dword[edx+%1.next],esi
		jmp	short %%not_first		; yes, set old.next to new
%%first:
		mov	dword[%2],esi			; no, make this the first element in the list
%%not_first:
		mov	dword[esi+%1.next],0		; this is the last element of the list

		xor	eax,eax				; return 0
%%quit:
		pop	edx

%endmacro
