; parallel.asm
;
; Robert stling, 2002-02-23
;
; Simple parallel port driver.
;
; To do: Add functions for other things than blinking LEDs :P


%define _parallel_asm 0

[bits 32]
[section .text]

%include "parallel.inc"

global	parallel_init
global	parallel_send
global	parallel_read

; parallel_init
;
; load parallel port base ports from BIOS data area
;
; returns:
; all registers unchanged

parallel_init:
		push	eax

		mov	dword[parallel_ports],0

		movzx	eax,word[0x408]		; is pp0 there?
		or	eax,eax
		jz	.done			; nope, quit

		mov	dword[parallel_base],eax
		inc	dword[parallel_ports]

		movzx	eax,word[0x40a]		; is pp1 there?
		or	eax,eax
		jz	.done			; nope, quit

		mov	dword[parallel_base+4],eax
		inc	dword[parallel_ports]

		movzx	eax,word[0x40c]		; is pp2 there?
		or	eax,eax
		jz	.done			; nope, quit

		mov	dword[parallel_base+8],eax
		inc	dword[parallel_ports]

.done:
		pop	eax
		ret



; parallel_send
;
; send a data byte to parallel port
;
; al = data byte
; ecx = port number (0 = pp0, 1 = pp1, 2 = pp2)
;
; returns:
; all registers unchanged

parallel_send:
		push	edx

		mov	edx,dword[parallel_base+(ecx*4)]
		out	dx,al

		pop	edx
		ret


; parallel_read
;
; read a data byte from parallel port
;
; ecx = port number (0 = pp0, 1 = pp1, 2 = pp2)
;
; returns:
; al = data byte

parallel_read:
		push	edx

		mov	edx,dword[parallel_base+(ecx*4)]
		in	al,dx

		pop	edx
		ret


[section .bss]

parallel_ports	resd	1
parallel_base	resd	3
