; pipe.asm
;
; Robert stling, 2002-05-03
;
; Pipes. :-)


%define _pipe_asm 0

[bits 32]
[section .text]

%include "config.inc"

%include "pipe.inc"
%include "mem.inc"

global pipe_create
global pipe_empty
global pipe_read
global pipe_write
global pipe_close


; pipe_create
;
; Creates a pipe.
;
; Returns:
; edi = pointer to the first pipe_block struct. (NULL if error)

pipe_create:
		push	eax
		push	esi

		mov	eax,pipe_block_size		; Allocate memory for a pipe_block struct
		call	mem_alloc
		or	esi,esi
		jz	.error

		mov	edi,esi
		mov	eax,pipe_blocksize		; Allocate a block of memory for it.
		call	mem_alloc
		jz	.error

		mov	dword[edi+pipe_block.ptr],esi
		mov	dword[edi+pipe_block.next],0
		mov	dword[edi+pipe_block.prev],0
		mov	dword[edi+pipe_block.inptr],0
		mov	dword[edi+pipe_block.outptr],0

		jmp	short .quit
.error:
		xor	edi,edi
.quit:
		pop	esi
		pop	eax
		ret



; pipe_empty
;
; ecx = pipe
;
; Returns:
; eax = 0 is it's not empty, 1 if it is.

pipe_empty:
		push	ecx

		mov	eax,dword[ecx+pipe_block.inptr]
		cmp	eax,dword[ecx+pipe_block.outptr]
		jz	.empty

		xor	eax,eax
		jmp	short .quit
.empty:
		mov	eax,1
.quit:
		pop	ecx
		ret



; pipe_read
;
; ecx = pointer to first pipe_block
;
; Returns:
; eax = 0 if a char is read, 1 if 
; ecx = pointer to first pipe_block (might have changed)
; dl = read char

pipe_read:
		mov	eax,dword[ecx+pipe_block.outptr]
		cmp	eax,dword[ecx+pipe_block.inptr]
		jz	.empty				; Check if the buffer is empty, if so, return error.

		mov	edx,dword[ecx+pipe_block.ptr]	; edx = data block
		mov	dl,byte[edx+eax]		; eax = pointer within the block
		inc	eax
		cmp	eax,pipe_blocksize		; Have we reached the end of this block?
		jb	.ok

		mov	eax,dword[ecx+pipe_block.ptr]	; eax points to the current memory block (no longer used)
		mov	ecx,dword[ecx+pipe_block.next]	; ecx now points to the new pipe_block struct
		mov	dword[ecx+pipe_block.prev],0
		call	mem_free			; free the old block
		xor	eax,eax

.ok:
		mov	dword[ecx+pipe_block.outptr],eax
		xor	eax,eax
		jmp	short .quit
.empty:
		mov	eax,1
.quit:
		ret



; pipe_write
;
; ecx = pointer to first pipe_block
; dl = byte to write
;
; Returns:
; eax = 0 if everything is OK.

pipe_write:
		push	ecx
		push	ebx
		push	esi
.find_last:
		cmp	dword[ecx+pipe_block.next],0
		jz	.found
		mov	ecx,dword[ecx+pipe_block.next]	; find the last block
		jmp	short .find_last
.found:

		mov	eax,dword[ecx+pipe_block.inptr]	; eax = input pointer
		mov	ebx,dword[ecx+pipe_block.ptr]	; ebx = pointer to data block
		mov	byte[ebx+eax],dl		; write the byte
		inc	eax
		cmp	eax,pipe_blocksize		; reached the end of this block?
		jb	.ok

		mov	eax,pipe_block_size
		call	mem_alloc
		or	esi,esi
		jz	.error

		mov	dword[ecx+pipe_block.inptr],pipe_blocksize
		mov	dword[ecx+pipe_block.next],esi
		mov	dword[esi+pipe_block.prev],ecx
		mov	ecx,esi
		mov	dword[ecx+pipe_block.next],0
		mov	dword[ecx+pipe_block.outptr],pipe_blocksize

		mov	eax,pipe_blocksize		; allocate a data block for the new struct
		call	mem_alloc
		or	esi,esi
		jz	.error

		mov	dword[ecx+pipe_block.ptr],esi
		xor	eax,eax

.ok:
		mov	dword[ecx+pipe_block.inptr],eax
		xor	eax,eax
		jmp	.quit
.error:
		mov	eax,1
.quit:
		pop	esi
		pop	ebx
		pop	ecx
		ret



; pipe_close
;
; Closes a pipe. 
;
; ecx = pointer to pipe_block struct

pipe_close:
		push	eax
		push	ecx
.again:
		mov	eax,dword[ecx+pipe_block.ptr]	; free the data block
		call	mem_free
		mov	eax,ecx
		mov	ecx,dword[ecx+pipe_block.next]
		call	mem_free			; free the pipe_block struct
		or	ecx,ecx
		jnz	.again				; if there's another struct, free that one too

		pop	ecx
		pop	eax
		ret
