; string.asm
;
; A simple library of string functions for Primula.
;
; 2002-05-09 Robert stling

%define _string_asm 0

global string_copy
global string_cmp
global string_cmp_s1
global string_lcase
global string_ucase
global string_hex_to_int
global string_dec_to_int
global string_int_to_hex


; string_copy
;
; esi = source
; edi = destination

string_copy:
		push	eax

.next:
		lodsb
		stosb
		or	al,al
		jnz	.next

		pop	eax
		ret



; string_cmp
;
; esi = string1
; edi = string2
;
; Returns:
; eax = 0 if equal

string_cmp:
		push	esi
		push	edi
.next:
		lodsb
		mov	ah,byte[edi]
		inc	edi
		cmp	al,ah
		jnz	.done
		or	al,al
		jz	.done
		jmp	short .next
.done:
		sub	al,ah
		movzx	eax,al

		pop	edi
		pop	esi
		ret



; string_cmp_s1
;
; If string2 is longer than string1, but the first part of it is equal to string1, the strings are concidered "equal".
;
; esi = string1
; edi = string2
;
; Returns:
; eax = 0 if equal

string_cmp_s1:
		push	esi
		push	edi
.next:
		lodsb
		mov	ah,byte[edi]
		inc	edi
		or	al,al
		jz	.equal
		cmp	al,ah
		jnz	.done
		jmp	short .next
.done:
		sub	al,ah
		movzx	eax,al
		jmp	short .quit
.equal:
		xor	eax,eax
.quit:
		pop	edi
		pop	esi
		ret



; ecx = number of digits to convert
; ebx = the number
; edi = location to store the string

string_int_to_hex:
		push	eax
.next:
		mov	al,bh
		and	al,0xf0
		shr	al,4

		sub	al,0x0a
		jc	.digit
		add	al,'A'
		jmp	short .char
.digit:
		add	al,'0'+0x0a
.char:
		stosb
		shl	ebx,4
		loop	.next
		pop	eax
		mov	byte[edi],0
		ret



; string_lcase
;
; Makes al lower-case

string_lcase:
		cmp	al,'A'
		jb	.done
		cmp	al,'Z'
		ja	.done
		add	al,'a'-'A'
.done:
		ret



; string_ucase
;
; Makes al upper-case

string_ucase:
		cmp	al,'a'
		jb	.done
		cmp	al,'z'
		ja	.done
		sub	al,'a'-'A'
.done:
		ret



; string_hex_to_int
;
; esi = string
;
; Returns:
; ebx = number
; eax = 0 if OK, else 1

string_hex_to_int:
		push	esi
		xor	ebx,ebx
.next:
		lodsb
		or	al,al
		jz	.finished
		call	string_ucase
		cmp	al,'A'
		jb	.not_char
		cmp	al,'F'
		ja	.not_char

		sub	al,'A'-0x0a
		jmp	short .done
.not_char:
		cmp	al,'0'
		jb	.error
		cmp	al,'9'
		ja	.error

		sub	al,'0'
.done:
		shl	ebx,4
		or	bl,al
		jmp	.next

.error:
		mov	eax,1
		jmp	short .quit
.finished:
		xor	eax,eax
.quit:
		pop	esi
		ret



; string_dec_to_int
;
; esi = string
;
; Returns:
; ebx = number
; eax = status (0 is OK, else 1)
; esi = pointer to the first char of the next string (the 'a' in '124a')

string_dec_to_int:
		push	ecx
		push	edx

		xor	ebx,ebx
.next:
		lodsb
		or	al,al
		jz	.finished
		cmp	al,'0'
		jb	.error
		cmp	al,'9'
		ja	.error

		sub	al,'0'
		movzx	ecx,al

		mov	eax,10
		mul	ebx
		xchg	eax,ebx

		add	ebx,ecx
		jmp	short .next

.finished:
		xor	eax,eax
		jmp	short .quit
.error:
		dec	esi
		mov	eax,1
.quit:
		pop	edx
		pop	ecx
		ret
