; timer.asm
;
; Robert stling, 2002-02-03
;
; Timer related issues, like calling proc_switch
;
; Robert stling, 2002-03-13:
; * Added callback features


%define _timer_asm 0

[bits 32]
[section .text]

global	timer_handler
global	timer_delay
global	timer_add_callback

%include "timer.inc"
%include "proc.inc"
%include "mem.inc"


; timer_add_callback
;
; Add a callback function
;
; edi = ptr to callback
;
; Returns:
; eax = 0 if OK, else 1

timer_add_callback:
		push	edx
		push	esi

		mov	eax,dword[timer_callbacks]	; eax points to the current element
		mov	edx,eax				; edx points to last element
.not_null:
		or	eax,eax				; have we found the last element of the list?
		jz	.is_null			; yeah
		mov	edx,eax				; nope, last element = current element
		mov	eax,dword[eax+timer_callback.next]
		jmp	short .not_null			; check the next element
.is_null:
		mov	eax,timer_callback_size		; allocate memory for the new element
		call	mem_alloc
		or	esi,esi				; did we fail?
		jnz	.mem_ok				; nope

		mov	eax,1				; yea, return 1
		jmp	.quit
.mem_ok:
		or	edx,edx				; was there any previous element?
		jz	.first				; nope, jump.
		mov	dword[edx+timer_callback.next],esi
		jmp	short .not_first		; yes, set old.next to new
.first:
		mov	dword[timer_callbacks],esi	; no, make this the first element in the list
.not_first:
		mov	dword[esi+timer_callback.next],0 ; this is the last element of the list
		mov	dword[esi+timer_callback.ptr],edi

		xor	eax,eax				; return 0
.quit:
		pop	esi
		pop	edx
		ret



; timer_delay
;
; ecx:eax = 64-bit value with the time to wait (in (10^-2)s units)
;
; returns:
; all registers unchanged

timer_delay:
		push	eax
		push	ecx

		add	eax,dword[timer_counter]
		adc	ecx,dword[timer_counter+4]
		; ecx:eax is now the time we should return

.not_yet:
		int	0x30
		cmp	dword[timer_counter+4],ecx
		jb	.not_yet		; if high dword is below, repeat

.not_really:
		cmp	dword[timer_counter],eax
		jb	.not_really		; wait while low word is still below

		pop	ecx
		pop	eax

		ret


; timer_handler
;
; returns:
; all registers unchanged

timer_handler:
						; increase the system counter value
		add	dword[timer_counter],byte 1
		adc	dword[timer_counter+4],byte 0

		cmp	byte[proc_mt],0		; multitasking?
		jz	.no_mt

		call	proc_switch		; yes, switch process
.no_mt:

		push	eax
		mov	eax,dword[timer_callbacks]
.not_null:
		or	eax,eax
		jz	.null

		call	dword[eax+timer_callback.ptr]
		mov	eax,dword[eax+timer_callback.next]
		jmp	short .not_null
.null:
		pop	eax

		ret


[section .data]

timer_counter	dd	0,0
timer_callbacks	dd	0
